@@ -1,9 +1,10 @@
 use anyhow::{bail, Context, Result};
 use byteorder::{NetworkEndian, WriteBytesExt};
+use std::cmp::max;
 use std::io::{Cursor, Read, Write};
 use std::net::{TcpStream, UdpSocket};
 use std::time::{Duration, Instant};
-use toluol::{DnsFlags, DnsMessage, DnsOpcode};
+use toluol::{DnsFlags, DnsMessage, DnsOpcode, DnsRecord};
 
 #[cfg(feature = "tls")]
 use {anyhow::anyhow, std::convert::TryInto, std::net::ToSocketAddrs, std::sync::Arc};
@@ -12,15 +13,17 @@ mod args;
 use args::ConnectionType;
 
 // TODO
+// - see if we can get nicer error messages
 // - add +trace support
 // - JSON output format
 // - DoH
 // - Colors for output
-// - add tests for parsing
+// - add tests for parsing (look at cargo fuzz)
+// - DNSSEC verification?
 
 fn main() -> Result<()> {
     let bufsize = 4096; // seems reasonable
-    let args = args::parse_args().context("Could not parse CLI arguments.")?;
+    let args = args::parse_args();
     let data = prepare_query(&args, bufsize)?;
 
     let (answer, bytes_recvd, elapsed) = match args.connection_type {
@@ -67,12 +70,10 @@ fn send_query_udp(
         .set_read_timeout(Some(Duration::new(10, 0)))
         .context("Could not set UDP socket read timeout.")?;
 
+    let addr = format!("{}:{}", args.nameserver, args.port);
     socket
-        .connect(format!("{}:53", args.nameserver))
-        .context(format!(
-            "Could not connect to {}:53 via UDP.",
-            args.nameserver
-        ))?;
+        .connect(&addr)
+        .context(format!("Could not connect to {} via UDP.", addr))?;
 
     let before = Instant::now();
     socket
@@ -94,9 +95,10 @@ fn send_query_tcp(
     bufsize: u16,
     data: &[u8],
 ) -> Result<(Vec<u8>, u16, Duration)> {
-    let mut socket = TcpStream::connect(format!("{}:53", args.nameserver)).context(format!(
-        "Could not connect to {}:53 via TCP, is the server running?",
-        args.nameserver
+    let addr = format!("{}:{}", args.nameserver, args.port);
+    let mut socket = TcpStream::connect(&addr).context(format!(
+        "Could not connect to {} via TCP, is the server running?",
+        addr
     ))?;
 
     socket
@@ -170,14 +172,14 @@ fn send_query_tls(args: &args::Args, data: &[u8]) -> Result<(Vec<u8>, u16, Durat
     let mut session = rustls::ClientConnection::new(Arc::new(config), nameserver)
         .context("Could not create TLS connection.")?;
 
-    let addr = format!("{}:853", args.nameserver)
+    let addr = format!("{}:{}", args.nameserver, args.port)
         .as_str()
         .to_socket_addrs()
         .context("Invalid nameserver address")?
         .next()
         .ok_or_else(|| anyhow!("Invalid nameserver address"))?;
     let mut socket = TcpStream::connect_timeout(&addr, Duration::new(10, 0)).context(format!(
-        "Failed to connect to {}:853, is the server configured to use DNS over TLS?",
+        "Failed to connect to {}, is the server configured to use DNS over TLS?",
         addr
     ))?;
     let mut plaintext = Vec::new();
@@ -242,18 +244,24 @@ fn send_query_tls(args: &args::Args, data: &[u8]) -> Result<(Vec<u8>, u16, Durat
 }
 
 fn display_result(res: &DnsMessage, args: &args::Args, bytes_recvd: u16, elapsed: &Duration) {
-    #[cfg(feature = "tls")]
-    let port = if args.connection_type == ConnectionType::TLS {
-        853
-    } else {
-        53
-    };
-    #[cfg(not(feature = "tls"))]
-    let port = 53;
-    println!("{}", res);
-    println!();
-    println!("Query metadata:");
-    println!("\tTime: {} ms", elapsed.as_millis());
-    println!("\tReply size: {} bytes", bytes_recvd);
-    println!("\tServer: {}:{}", args.nameserver, port);
+    if !args.short {
+        println!("{}", res);
+        println!();
+        println!("Query metadata:");
+        println!("\tTime: {} ms", elapsed.as_millis());
+        println!("\tReply size: {} bytes", bytes_recvd);
+        println!("\tServer: {}:{}", args.nameserver, args.port);
+        return;
+    }
+
+    let (mut max_owner_len, mut max_type_len) = (0, 0);
+    for answer in &res.answers {
+        if let DnsRecord::NONOPT { name, atype, .. } = answer {
+            max_owner_len = max(max_owner_len, name.len());
+            max_type_len = max(max_type_len, atype.to_string().len());
+        }
+    }
+    for answer in &res.answers {
+        println!("{}", answer.as_padded_string(max_owner_len, max_type_len));
+    }
 }
